#include "parallel.h"
#include "interface.h"

void main(int argc, char *argv[])
{
   ZIP_MAILER       *Mailer;

   matrix_host_bin   Matrix;
   vector_host_bin   Vector;

   int               p;
   int               q;

   int               m;
   int               n;

   long              sec;
   long              usec;
   int               i;

   short            *IBUSO = NULL;
   int               NBUS;

   printf("cube host -- demo for parallel sparse LU solver\n");

   initialize_host(NODE_PGM_NAME, &Matrix, &Vector);

   /* Hosue keeping work */

   Mailer = Matrix.Mailer;

   read_order("Order0.dat", &IBUSO, &NBUS);
   print("NBUS = %d\n", NBUS);

   g2_pq(Mailer, p, q);

   Matrix.M = NBUS * 2;
   set_distribution(Mailer, Matrix.Row_Dis_Type, CDISTRI_ROW,
                    &(Matrix.Row_Dis),
                    &(Matrix.Row_Distribution),
                    &(Matrix.Row_Inverse_Distribution),
                    &(Matrix.Row_Distribution_limit),
                    &(Matrix.Row_Distribution_init),
                    Matrix.P, p, Matrix.M, &m,
                    (int) Matrix.Row_Extra_param1,
                    (int) Matrix.Row_Extra_param2,
                    &(Matrix.Row_Extra));

   Matrix.N = NBUS * 2;
   set_distribution(Mailer, Matrix.Col_Dis_Type, CDISTRI_COLUMN,
                    &(Matrix.Col_Dis),
                    &(Matrix.Col_Distribution),
                    &(Matrix.Col_Inverse_Distribution),
                    &(Matrix.Col_Distribution_limit),
                    &(Matrix.Col_Distribution_init),
                    Matrix.Q, q, Matrix.N, &n,
                    (int) Matrix.Col_Extra_param1,
                    (int) Matrix.Col_Extra_param2,
                    &(Matrix.Col_Extra));

   Matrix.NBUS = NBUS;
   if ((Matrix.Row_major_LU = (element **) malloc(sizeof(element *) * NBUS))
       == NULL)
   {
      print("parallel_cube_host(): Out of memory!\n");
      exit(1);
   }

   if ((Matrix.Row_major_LU_element_Count = (int *) malloc(sizeof(int)
                                                           * NBUS)) == NULL)
   {
      print("parallel_cube_host(): Out of memory!\n");
      exit(1);
   }

   for (i = 0; i < NBUS; i++)
   {
      Matrix.Row_major_LU[i] = NULL;
      Matrix.Row_major_LU_element_Count[i] = 0;
   }

   Vector.M = Matrix.M;
   Vector.N = Matrix.N;

   Vector.Row_Dis = Matrix.Row_Dis;
   Vector.Row_Distribution = Matrix.Row_Distribution;
   Vector.Row_Inverse_Distribution = Matrix.Row_Inverse_Distribution;
   Vector.Row_Distribution_limit = Matrix.Row_Distribution_limit;
   Vector.Row_Extra = Matrix.Row_Extra;

   Vector.Col_Dis = Matrix.Col_Dis;
   Vector.Col_Distribution = Matrix.Col_Distribution;
   Vector.Col_Inverse_Distribution = Matrix.Col_Inverse_Distribution;
   Vector.Col_Distribution_limit = Matrix.Col_Distribution_limit;
   Vector.Col_Extra = Matrix.Col_Extra;

   Vector.NBUS = NBUS;

   read_matrix("Matrix1.0.dat", &Matrix);
   transmit_matrix_host(Mailer, &Matrix);
   clear_real_matrix_host(&Matrix);

   receive_timing_host(Mailer, &sec, &usec);
   print("Timing Data for LU factor: %ld sec %ld usec\n", sec, usec);

   read_vector("RHS1.1.1.dat", &Vector);
   transmit_col_vector_host(Mailer, &Vector);
   clear_col_vector_host(&Vector);

   receive_row_vector_host(Mailer, &Vector);
   write_vector("Sol1.1.1.dat", &Vector);
   clear_row_vector_host(&Vector);

   receive_timing_host(Mailer, &sec, &usec);
   print("Timing Data for forward and back solve: %ld sec %ld usec\n",
         sec, usec);

   zip_exit();
   system("freecube");
}

